@@ -5,15 +5,19 @@ https://github.com/Ernst79/bleparser/blob/c42ae922e1abed2720c7fac993777e1bd59c0c
 
 MIT License applies.
 """
+
 from __future__ import annotations
 
 import logging
 import struct
+from dataclasses import dataclass
+from enum import Enum
 
 from bluetooth_data_tools import short_address
 from bluetooth_sensor_state_data import BluetoothData
+from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
 from home_assistant_bluetooth import BluetoothServiceInfo
-from sensor_state_data import SensorLibrary
+from sensor_state_data import BinarySensorDeviceClass, SensorLibrary
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -103,6 +107,60 @@ def decode_pm25_from_4_bytes(packet_value: int) -> int:
     return int(packet_value % 1000)
 
 
+def calculate_crc(data: bytes) -> int:
+    crc = 0x1D0F
+    for b in data:
+        for s in range(7, -1, -1):
+            mask = 0
+            if (crc >> 15) ^ (b >> s) & 1:
+                mask = 0x1021
+            crc = ((crc << 1) ^ mask) & 0xFFFF
+    return crc
+
+
+def decrypt_data(key: bytes, data: bytes) -> bytes:
+    cipher = Cipher(algorithms.AES(key[::-1]), modes.ECB())
+    decryptor = cipher.decryptor()
+    return (decryptor.update(data[::-1]) + decryptor.finalize())[::-1]
+
+
+class SensorType(Enum):
+    THERMOMETER = "thermometer"
+    BUTTON = "button"
+    MOTION = "motion"
+    WINDOW = "window"
+    VIBRATION = "vibration"
+    PRESENCE = "presence"
+
+
+@dataclass
+class ModelInfo:
+    """Model information for Govee sensors."""
+
+    model_id: str
+    sensor_type: SensorType
+    button_count: int
+    sleepy: bool
+
+
+_MODEL_DB = {
+    "H5121": ModelInfo("H5121", SensorType.MOTION, 0, True),
+    "H5122": ModelInfo("H5122", SensorType.BUTTON, 1, True),
+    "H5123": ModelInfo("H5123", SensorType.WINDOW, 0, True),
+    "H5124": ModelInfo("H5124", SensorType.VIBRATION, 0, True),
+    "H5125": ModelInfo("H5125", SensorType.BUTTON, 6, True),
+    "H5126": ModelInfo("H5126", SensorType.BUTTON, 2, True),
+    "H5127": ModelInfo("H5127", SensorType.PRESENCE, 0, True),
+}
+
+
+def get_model_info(model_id: str) -> ModelInfo:
+    """Get model information for a Govee sensor."""
+    return _MODEL_DB.get(
+        model_id, ModelInfo(model_id, SensorType.THERMOMETER, 0, False)
+    )
+
+
 class GoveeBluetoothDeviceData(BluetoothData):
     """Data for Govee BLE sensors."""
 
@@ -127,6 +185,35 @@ class GoveeBluetoothDeviceData(BluetoothData):
                 continue
             self._process_mfr_data(address, local_name, mfr_id, mfr_data, service_uuids)
 
+    @property
+    def device_type(self) -> str | None:
+        """Return the device type."""
+        primary_device_id = self.primary_device_id
+        if device_type := self._device_id_to_type.get(primary_device_id):
+            return device_type.partition("-")[0]
+        return None
+
+    @property
+    def button_count(self) -> int:
+        """Return the number of buttons on the device."""
+        device_type = self.device_type
+        assert device_type is not None
+        return get_model_info(device_type).button_count
+
+    @property
+    def sensor_type(self) -> SensorType:
+        """Return the sensor type."""
+        device_type = self.device_type
+        assert device_type is not None
+        return get_model_info(device_type).sensor_type
+
+    @property
+    def sleepy(self) -> bool:
+        """Return if the device is sleep."""
+        device_type = self.device_type
+        assert device_type is not None
+        return get_model_info(device_type).sleepy
+
     def _process_mfr_data(
         self,
         address: str,
@@ -144,7 +231,101 @@ class GoveeBluetoothDeviceData(BluetoothData):
             data = data[:-25]
             msg_length = len(data)
             if debug_logging:
-                _LOGGER.debug("Cleaned up packet: %s %s", mgr_id, hex(data))
+                _LOGGER.debug("Cleaned up packet: %s %s", mgr_id, data.hex())
+
+        if msg_length == 24:
+            time_ms = data[2:6]
+            enc_data = data[6:22]
+            enc_crc = data[22:24]
+            if not calculate_crc(enc_data) == int.from_bytes(enc_crc, "big"):
+                _LOGGER.debug("CRC check failed for H512x: %s", hex(data))
+                return
+
+            key = time_ms + bytes(12)
+            try:
+                decrypted = decrypt_data(key, enc_data)
+            except ValueError:
+                _LOGGER.warning("Failed to decrypt H512x: %s", hex(data))
+                return
+            _LOGGER.debug("Decrypted H512x: %s - %s", decrypted, decrypted.hex())
+            model_id = decrypted[2]
+            # GV5121
+            # 01040302640100000000000000000000
+
+            # GV5122
+            # 01050802640000000000000000000000
+            # 01050802640000000000000000000000
+
+            # GV5123
+            # 01050202640200000000000000000000
+
+            # GV5124
+            # 01030902640100000000000000000000
+
+            # GV5125
+            # 01010a02640000000000000000000000
+
+            # GV5126
+            # 01010b02640100000000000000000000
+            sensor_type = SensorType.BUTTON
+            if "GV5121" in local_name or model_id == 3:
+                self.set_device_type("H5121")
+                self.set_device_name(f"5121{short_address(address)}")
+                sensor_type = SensorType.MOTION
+            elif "GV5122" in local_name or model_id == 8:
+                self.set_device_type("H5122")
+                self.set_device_name(f"5122{short_address(address)}")
+            elif "GV5123" in local_name or model_id == 2:
+                self.set_device_type("H5123")
+                self.set_device_name(f"5123{short_address(address)}")
+                sensor_type = SensorType.WINDOW
+            elif "GV5124" in local_name or model_id == 9:
+                self.set_device_type("H5124")
+                self.set_device_name(f"5124{short_address(address)}")
+                sensor_type = SensorType.VIBRATION
+            elif "GV5125" in local_name or model_id == 10:
+                self.set_device_type("H5125")
+                self.set_device_name(f"5125{short_address(address)}")
+            elif "GV5126" in local_name or model_id == 11:
+                self.set_device_type("H5126")
+                self.set_device_name(f"5126{short_address(address)}")
+            else:
+                return
+
+            battery_percentage = decrypted[4]
+            button_number_pressed = decrypted[5]
+            self.update_predefined_sensor(
+                SensorLibrary.BATTERY__PERCENTAGE, battery_percentage
+            )
+            if sensor_type is SensorType.WINDOW:
+                # H5123 is a door/window sensor
+                self.update_predefined_binary_sensor(
+                    BinarySensorDeviceClass.WINDOW, button_number_pressed == 2
+                )
+            elif sensor_type is SensorType.VIBRATION:
+                # H5124 is a vibration sensor
+                if button_number_pressed == 1:
+                    self.fire_event("vibration", "vibration")
+            elif sensor_type is SensorType.MOTION:
+                if button_number_pressed == 1:
+                    self.fire_event("motion", "motion")
+            else:
+                self.fire_event(f"button_{button_number_pressed}", "press")
+            return
+
+        if msg_length == 6 and (
+            data.startswith(b"\xec\x00\x01\x01")
+            and "H5127" in local_name
+            or mgr_id == 0x8803
+        ):
+            self.set_device_type("H5127")
+            self.set_device_name(f"H5127{short_address(address)}")
+            present = data[4] == 1
+            motion = data[5] == 17
+            self.update_predefined_binary_sensor(
+                BinarySensorDeviceClass.OCCUPANCY, present
+            )
+            self.update_predefined_binary_sensor(BinarySensorDeviceClass.MOTION, motion)
 
         if msg_length == 6 and (
             (is_5072 := "H5072" in local_name)
